#ifndef DPX_DEVICE_API_H
#define DPX_DEVICE_API_H

#include <stdio.h>

/*
  Forward declarations of private structures
*/
struct _DPX_TOUCH;
typedef struct _DPX_TOUCH DPX_TOUCH, *PDPX_TOUCH;

struct _DPX_DEVICE;
typedef struct _DPX_DEVICE DPX_DEVICE, *PDPX_DEVICE;

struct _DPX_CONTEXT;
typedef struct _DPX_CONTEXT DPX_CONTEXT, *PDPX_CONTEXT;

struct _DPX_CALIBRATION_ZONE;
typedef struct _DPX_CALIBRATION_ZONE DPX_CALIBRATION_ZONE, *PDPX_CALIBRATION_ZONE;

#define DPX_CALIBRATION_ZONE_SIZE 360

/*
  Structure the describes device status
*/
typedef struct _DPX_DEVICE_STATUS
{
    int Enabled;
    int FrameRate;
    int FlipVertical;
    int FlipHorizontal;
    int Gain;
    int TouchPoints;
    int VersionMajor;
    int VersionMinor;
    int ControllerPosition;
    int AdvancedCalibration;
    int LogicalLeft;
    int LogicalRight;
    int LogicalTop;
    int LogicalBottom;
    int NumCols;
    int NumRows;
    int DisplayOrientation;
    unsigned char Serial[32];

    int PalmRejection;
    int wFalseTouchsAG;
    int DriverVersion;
    int EnabledCols[64];
    int EnabledRows[64];
    int PhysicalTop;
    int PhysicalBottom;
    int PhysicalLeft;
    int PhysicalRight;
    int Orientation;
} DPX_DEVICE_STATUS, *PDPX_DEVICE_STATUS;

/*
    Name: DPXContextCreate
    Description: Creates a context for device consulting and interaction
    Parameters: None
    Return value: Returns a pointer to a DPX_CONTEXT
*/
PDPX_CONTEXT DPXContextCreate();

/*
    Name: DPXContextDestroy
    Description: Releases all the allocated objects and memory necessary for context
    Parameters: DPX_CONTEXT pointer
    Return value: None
*/
void DPXContextDestroy(PDPX_CONTEXT context);

/*
    Name: DPXContextHasDevice
    Description: Determines if a device with the speficied DEVICE_ID is currently connected. Returns true if found, otherwise returns false.
    Parameters: DPX_CONTEXT pointer
    Return value: 1 if present, 0 if not
*/
int DPXContextHasDevice(PDPX_CONTEXT context, const char* deviceID);

/*
    Name: DPXContextUpdateDevices
    Description: Updates the internal devices references. It should be called regulary to make sure the count and references are updated.
    Parameters: DPX_CONTEXT pointer
    Return value: void
*/
void DPXContextUpdateDevices(PDPX_CONTEXT context);

/*
    Name: DPXContextGetDeviceCount
    Description: Returns a number of connected devices. Requires context run loop running.
    Parameters: DPX_CONTEXT pointer
    Return value: number of present device
*/
int DPXContextGetDeviceCount(PDPX_CONTEXT context);

/*
    Name: DPXContextGetDevice
    Description: Returns a DPX_DEVICE pointer for the speficied device index. Index value must be between 0 and DPXContextGetDeviceCount-1
    Parameters: DPX_CONTEXT pointer, int index
    Return value: DPX_DEVICE pointer
*/
PDPX_DEVICE DPXContextGetDevice(PDPX_CONTEXT context, int index);

/*
    Name: DPXContextGetDeviceWithSerial
    Description: Returns a DPX_DEVICE pointer for the speficied device serial if present (call DPXContextUpdateDevices before).
    Parameters: DPX_CONTEXT pointer, char *
    Return value: DPX_DEVICE pointer
*/
PDPX_DEVICE DPXContextGetDeviceWithSerial(PDPX_CONTEXT Context, const char *Serial);

/*
    Name: DPXDeviceOpen
    Description: Initializes and opens a device
    Parameters: DPX_DEVICE pointer
    Return value: 1 on success, 0 failure
*/
int DPXDeviceOpen(PDPX_DEVICE device);

/*
    Name: DPXDeviceLoadSettings
    Description: Loads device settings from disk persisted state
    Parameters: DPX_DEVICE pointer
    Return value: 1 on success, 0 failure
*/
int DPXDeviceLoadSettings(PDPX_DEVICE device);

/*
    Name: DPXDeviceClose
    Description: Closes device
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceClose(PDPX_DEVICE device);

/*
    Name: DPXDeviceGetSerial
    Description: Returns Device Serial
    Parameters: DPX_DEVICE pointer
    Return value: const char* with serial number
*/
const char* DPXDeviceGetSerial(PDPX_DEVICE device);

/*
    Name: DPXDeviceLock
    Description: Locks the mutex associated with the device, for safe device operations
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceLock(PDPX_DEVICE device);

/*
    Name: DPXDeviceUnlock
    Description: Unlocks the mutex associated with the device
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceUnlock(PDPX_DEVICE device);

/*
    Name: DPXDeviceGetStatus
    Description: Writes the status of a determined device in the provided DPX_DEVICE_STATUS structure pointer
    Parameters: DPX_DEVICE pointer, DPX_DEVICE_STATUS
    Return value: 1 on success, 0 failure
*/
int DPXDeviceGetStatus(PDPX_DEVICE device, PDPX_DEVICE_STATUS status);

/*
    Name: DPXDeviceSetEnabled
    Description: Sets touch injection on the operating system on or off.
                 Possible values are 0 or 1.
    Parameters: DPX_DEVICE pointer, (int) value
    Return value: None
*/
void DPXDeviceSetEnabled(PDPX_DEVICE device, int value);

/*
    Name: DPXDeviceSetGain
    Description: Sets controller gain value. This is an advanced. Use carefully as it can bring bad touch experience.
                 Possible values are compreended between 32 (high) and 92 (low). Keep this value as higher as possible. Default is 48
    Parameters: DPX_DEVICE pointer, (int) value
    Return value: None
*/
void DPXDeviceSetGain(PDPX_DEVICE device, int value);

/*
    Name: DPXDeviceCalibrateFull
    Description: Requests a full electric calibration of the controller.
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceCalibrateFull(PDPX_DEVICE device);

/*
    Name: DPXDeviceCalibratePartial
    Description: Requests a partial electric calibration of the controller.
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceCalibratePartial(PDPX_DEVICE device);

/*
    Name: DPXDeviceSetTestMode
    Description: Requests the device to enter in test mode.
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceSetTestMode(PDPX_DEVICE device);

/*
    Name: DPXDeviceClearTestMode
    Description: Requests the device to exit test mode.
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceClearTestMode(PDPX_DEVICE device);

/*
    Name: DPXDeviceLoadStateFromFile
    Description: Loads a device state from the specified file path to the driver.
                 Returns true if operation was successfull, otherwise returns false.
    Parameters: DPX_DEVICE pointer, const char* path
    Return value: 1 on success, 0 failure
*/
int DPXDeviceLoadStateFromFile(PDPX_DEVICE device, const char* path);

/*
    Name: DPXDeviceSaveStateToFile
    Description: Saves the current device state to the specified file path.
                 Returns true if operation was successfull, otherwise returns false.
    Parameters: DPX_DEVICE pointer, const char* path
    Return value: 1 on success, 0 failure
*/
int DPXDeviceSaveStateToFile(PDPX_DEVICE device, const char* path);

/*
    Name: DPXDeviceCopyFrame
    Description: Copies a device raw frame into the specified pointer.
                 Device raw frame is delivered on a unsigned char array
                 The array must have 4096 bytes length for full frame copy. Length must be less or equal to 4096.
                 Returns true if a new frame was successfully copied.
    Parameters: DPX_DEVICE pointer, unsigned char pointer, int length
    Return value: 1 on success, 0 failure
*/
int DPXDeviceCopyFrame(PDPX_DEVICE device, unsigned char *frame, int length);

/*
    Name: DPXDeviceResetGeometricCalibration
    Description: Reset geometric calibration
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceResetGeometricCalibration(PDPX_DEVICE device);

/*
    Name: DPXDeviceSaveGeometricCalibration
    Description: Set a new geometric calibration into the device
    Parameters: DPX_DEVICE pointer, DPX_CALIBRATION_ZONE pointer, Number of calibration zones
    Return value: None
*/
void DPXDeviceSaveGeometricCalibration(PDPX_DEVICE device, PDPX_CALIBRATION_ZONE zones, int numZones, int orientation, int flipX, int flipY);


#endif // DPX_DEVICE_API_H
