#ifndef DEVICE_H
#define DEVICE_H

#ifdef CONTROLPANEL
#include "DeviceAPI.h"
#endif

#include "Acquisition.h"
#include "Block.h"
#include "BlobDetection.h"
#include "Processing.h"
#include "Track.h"
#include "Tracking.h"

#ifdef LINUX
#include <pthread.h>
#endif

#ifdef WINDOWS
#include <windows.h>
#endif

#ifdef MACOSX
#include <pthread.h>
#include <CoreFoundation/CoreFoundation.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>
#include <mach/mach.h>
#define BUFFER_SIZE 512
typedef struct
{
    PDPX_DEVICE device;
    unsigned char bufferIndex;
    unsigned char buffer[BUFFER_SIZE];

} ReadBuffer;
#define kNumConcurrentReads 2
#endif

typedef struct _DPX_REPORTING_STATE
{
    int CurrentTouch;
    int ReportedTouches;
} DPX_REPORTING_STATE, *PDPX_REPORTING_STATE;

typedef struct _DEVICE_STATE
{
    DPX_ACQUISITION_STATE Acquisition;
    DPX_PROCESSING_STATE Processing;
    DPX_TRACKING_STATE Tracking;
    DPX_REPORTING_STATE Reporting;

    DPX_TOUCH Touches[DPX_TOUCH_MAX_COUNT];

    int EnabledCols[DPX_FRAME_MAX_COLS];
    int EnabledRows[DPX_FRAME_MAX_ROWS];

    int PhysicalTop;
    int PhysicalBottom;
    int PhysicalLeft;
    int PhysicalRight;

    int LogicalTop;
    int LogicalBottom;
    int LogicalLeft;
    int LogicalRight;

    int Enabled;
    int PalmRejection;

    int Connected;
    int NumCols;
    int NumRows;
    int AutoGainValue;
    int AutoAdjusting;

    int DisplayOrientation;
    int Orientation;
    int FlipX;
    int FlipY;
    int padding;

    int wFalseTouchsAG;

} DPX_DEVICE_STATE, *PDPX_DEVICE_STATE;

typedef struct _DPX_DEVICE_DATA
{
    DPX_ACQUISITION_DATA Acquisition;
    DPX_PROCESSING_DATA Processing;
    DPX_BLOBDETECTION_DATA BlobDetection;
    DPX_TRACKING_DATA Tracking;
} DPX_DEVICE_DATA, *PDPX_DEVICE_DATA;

typedef struct _DPX_CONTROL_REQUEST
{
    unsigned char bRequest;
    unsigned short wValue;
    unsigned short wIndex;
} DPX_CONTROL_REQUEST, *PDPX_CONTROL_REQUEST;

struct _DPX_DEVICE
{
    #ifdef CONTROLPANEL
    #ifdef WINDOWS
    CRITICAL_SECTION Mutex;
    HANDLE Thread;
    CONDITION_VARIABLE OpenCond;
    CONDITION_VARIABLE StateCond;
    #else
    pthread_mutex_t Mutex;
    pthread_t Thread;
    pthread_cond_t StateCond;
    pthread_cond_t OpenCond;
    #endif
    #endif

    int Running;
    int Inited;
    int Index;
    int Pid;
    char* Id;
    char* Serial;
    void* Handle;
    PDPX_CONTEXT Context;
    char StateValid;

    void* StateReadHandle;
    void* StateWriteHandle;
    void* BulkHandle;
    char* deviceString;
    char* deviceState;
    char* deviceBulk;

    #if defined(LINUX)
    struct libusb_transfer *transfer1, *transfer2;
    #endif

    #if defined(MACOSX)
    ReadBuffer readBuffer[kNumConcurrentReads];
    IOUSBDeviceInterface245 **deviceInterface;
    IOUSBInterfaceInterface245 **usbInterface;
    #endif

    int ShouldNotifyCallback;

    DPX_DEVICE_DATA Data;
    DPX_DEVICE_STATE State;
};

void DPXDeviceInit(PDPX_DEVICE device);
void DPXDeviceSetConnectionState(PDPX_DEVICE device, int state);
void DPXDeviceStateChangeBegin(PDPX_DEVICE device);
void DPXDeviceStateChangeEnd(PDPX_DEVICE device);
void DPXDeviceSetPhysicalTop(PDPX_DEVICE device, int value);
void DPXDeviceSetPhysicalBottom(PDPX_DEVICE device, int value);
void DPXDeviceSetPhysicalLeft(PDPX_DEVICE device, int value);
void DPXDeviceSetPhysicalRight(PDPX_DEVICE device, int value);

void DPXDeviceSetLogicalTop(PDPX_DEVICE device, int value);
void DPXDeviceSetLogicalBottom(PDPX_DEVICE device, int value);
void DPXDeviceSetLogicalLeft(PDPX_DEVICE device, int value);
void DPXDeviceSetLogicalRight(PDPX_DEVICE device, int value);

void DPXDeviceSetPalmRejection(PDPX_DEVICE device, int value);

void DPXDeviceSetDisplayOrientation(PDPX_DEVICE device, int value);

void DPXDeviceSetAutoAdjusting(PDPX_DEVICE device, int value);
void DPXDeviceSetWFalseTouchsAG(PDPX_DEVICE device, int value);
void DPXDeviceFillRequestPacket(PDPX_DEVICE device, unsigned char * requestPacket, unsigned char request);

int DPXDeviceSendBulkRequest(PDPX_DEVICE device, unsigned char targetEndpoint, unsigned char *buffer, int bufferSize, int *transferedBytes, int timeout);
int DPXDeviceSendControlRequest(PDPX_DEVICE device, unsigned char request, unsigned short value, unsigned short index);
int DPXDeviceSetState(PDPX_DEVICE device, PDPX_DEVICE_STATE state);
int DPXDeviceSetState_DeviceOnly(PDPX_DEVICE device, PDPX_DEVICE_STATE state);
int DPXDeviceGetState(PDPX_DEVICE device);

int DPXDeviceOpenImpl(PDPX_DEVICE device);
void DPXDeviceCloseImpl(PDPX_DEVICE device);

void DPXDeviceReportTouch(PDPX_DEVICE device);

void DPXDevicePerformPartialCalibrationCheck(void* param);
void DPXDevicePerformFalseTouchesCheck(void* param);
int DPXDeviceProcessFrame(void *);

#ifndef WINDOWS_DRIVER
bool DPXDeviceStateChangeEnd_DeviceOnly( PDPX_DEVICE device );
bool DPXDeviceLoadSettings_ReadOnly( PDPX_DEVICE device );
bool DPXReadStateFromFile( PDPX_DEVICE_STATE state, const char *path );
bool DPXWriteStateToDevice( PDPX_DEVICE device, PDPX_DEVICE_STATE state );
#endif

#endif // DEVICE_H
